# coding:utf-8
import os
import re
from distutils import util

from flask import Markup


dict_status = {"ok": "success"}
label_status_translation = {
    "ok": "success",
    "error": "important",
    "in-progress": "info",
}


def asbool(s):
    """ Return the boolean value ``True`` if the case-lowered value of string
    input ``s`` is any of ``t``, ``true``, ``y``, ``on``, or ``1``, otherwise
    return the boolean value ``False``. If ``s`` is the value ``None``,
    return ``False``. If ``s`` is already one of the boolean values ``True``
    or ``False``, return it."""
    return bool(util.strtobool(s))


def clean_uri(text):
    if text.startswith("http"):
        return Markup(text)
    else:
        return Markup(os.path.basename(text))


def utility_processor():
    def default(value, default):
        return value or default

    def trans_status(status, to_label=False):
        status = status.lower().replace(" ", "-")
        if asbool(to_label):
            return label_status_translation.get(status, status)
        return dict_status.get(status, status)

    return dict(trans_status=trans_status, default=default)



# coding: utf-8
from flask import (
    g,
    Blueprint,
    render_template,
    current_app,
    request,
    url_for,
    redirect,
    session,
)

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

import packtools
from .forms import XMLUploadForm
from .utils import analyze_xml


main = Blueprint("main", __name__)


@main.before_app_request
def add_context_settings():
    setattr(
        g,
        "SETTINGS_MAX_UPLOAD_SIZE",
        current_app.config.get("SETTINGS_MAX_UPLOAD_SIZE"),
    )
    setattr(g, "PACKTOOLS_VERSION", current_app.config.get("PACKTOOLS_VERSION"))


@main.route("/", defaults={"path_file": ""})
@main.route("/<path:path_file>")
def packtools_home(path_file):
    if not path_file:
        return redirect(url_for("main.packtools_stylechecker"))

    url_session = session.get("url_static_file", "http://localhost")
    return redirect(urljoin(url_session, path_file))


@main.route("/stylechecker", methods=["GET", "POST"])
def packtools_stylechecker():

    form = XMLUploadForm()
    context = dict(form=form)
    if form.validate_on_submit():

        if form.add_scielo_br_rules.data:
            extra_sch = packtools.catalogs.SCHEMAS["scielo-br"]
        else:
            extra_sch = None

        results, exc = analyze_xml(form.file.data, extra_schematron=extra_sch)
        context["results"] = results
        context["xml_exception"] = (
            exc and getattr(exc, "message", getattr(exc, "msg", str(exc))) or None
        )

    return render_template("validator/stylechecker.html", **context)


@main.route("/previews", methods=["GET", "POST"])
def packtools_preview_html():

    form = XMLUploadForm()
    context = dict(form=form)
    if form.validate_on_submit():

        session["url_static_file"] = form.url_static_file.data
        previews = []
        try:
            for lang, html_output in packtools.HTMLGenerator.parse(
                form.file.data,
                valid_only=False,
                css=url_for("static", filename="css/htmlgenerator/scielo-article.css"),
                print_css=url_for(
                    "static", filename="css/htmlgenerator/scielo-bundle-print.css"
                ),
                js=url_for("static", filename="js/htmlgenerator/scielo-article-min.js"),
            ):
                previews.append({"lang": lang, "html": html_output})
        except Exception as e:
            # print(e.message)
            # qualquer exeção aborta a pre-visualização mas continua com o resto
            previews = []

        context["previews"] = previews

    return render_template("validator/preview_html.html", **context)



# coding: utf-8
from __future__ import print_function, unicode_literals
import argparse
import sys
import pkg_resources
import logging

from lxml import etree

import packtools
from packtools import catalogs

LOGGER = logging.getLogger(__name__)


class XMLError(Exception):
    """ Represents errors that would block HTMLGenerator instance from
    being created.
    """


def get_htmlgenerator(
    xmlpath, no_network, no_checks, css, print_css, js, permlink, url_article_page, url_download_ris
):
    try:
        parsed_xml = packtools.XML(xmlpath, no_network=no_network)
    except IOError as e:
        raise XMLError('Error reading %s. Make sure it is a valid file-path or URL.' % xmlpath)
    except etree.XMLSyntaxError as e:
        raise XMLError('Error reading %s. Syntax error: %s' % (xmlpath, e))

    try:
        valid_only = not no_checks
        generator = packtools.HTMLGenerator.parse(
            parsed_xml, valid_only=valid_only, css=css,
            print_css=print_css, js=js,
            permlink=permlink,
            url_article_page=url_article_page,
            url_download_ris=url_download_ris)
    except ValueError as e:
        raise XMLError('Error reading %s. %s.' % (xmlpath, e))

    return generator


@packtools.utils.config_xml_catalog
def main():

    packtools_version = pkg_resources.get_distribution('packtools').version

    parser = argparse.ArgumentParser(description='HTML generator cli utility')
    parser.add_argument('--nonetwork', action='store_true',
                        help='prevents the retrieval of the DTD through the network')
    parser.add_argument('--nochecks', action='store_true',
                        help='prevents the validation against SciELO PS spec')
    parser.add_argument('--css', default=catalogs.HTML_GEN_DEFAULT_CSS_PATH,
                        help='URL or full path of the CSS file to use with generated htmls')
    parser.add_argument('--print_css', default=catalogs.HTML_GEN_DEFAULT_PRINT_CSS_PATH,
                        help='URL or full path of the CSS (media: print) file to use with generated htmls')
    parser.add_argument('--js', default=catalogs.HTML_GEN_DEFAULT_JS_PATH,
                        help='URL or full path of the JS file to use with generated htmls')
    parser.add_argument('--permlink', default='',
                        help='Permanente URL to access the article')
    parser.add_argument('--url_article_page', default='',
                        help='OPAC URL to access the article')
    parser.add_argument('--url_download_ris', default='',
                        help='URL to download RIS file (how to cite this article)')
    parser.add_argument('XML', nargs='+',
                        help='filesystem path or URL to the XML')
    parser.add_argument('--version', action='version', version=packtools_version)
    parser.add_argument('--loglevel', default='WARNING')
    args = parser.parse_args()

    logging.basicConfig(level=getattr(logging, args.loglevel.upper()))

    print('Please wait, this may take a while...', file=sys.stderr)

    for xml in packtools.utils.flatten(args.XML):
        LOGGER.info('starting generation of %s' % (xml,))

        try:
            html_generator = get_htmlgenerator(
                xml, args.nonetwork, args.nochecks,
                args.css, args.print_css, args.js,
                args.permlink, args.url_article_page, args.url_download_ris)
            LOGGER.debug('HTMLGenerator repr: %s' % repr(html_generator))
        except XMLError as e:
            LOGGER.debug(e)
            LOGGER.warning('Error generating %s. Skipping. Run with DEBUG for more info.', xml)
            continue

        try:
            for lang, trans_result in html_generator:
                fname, fext = xml.rsplit('.', 1)
                out_fname = '.'.join([fname, lang, 'html'])

                with open(out_fname, 'wb') as fp:
                    fp.write(etree.tostring(trans_result, pretty_print=True,
                                            encoding='utf-8', method='html',
                                            doctype=u"<!DOCTYPE html>"))

                print('Generated HTML file:', out_fname)
        except TypeError as e:
            LOGGER.debug(e)
            LOGGER.warning('Error generating %s. Skipping. Run with DEBUG for more info.', xml)
            continue

        LOGGER.info('finished generating %s' % (xml,))


if __name__ == '__main__':
    main()



# coding:utf-8
"""The domain-specific adapters.

An adapter is an object that provides domain-specific apis for another object,
called adaptee.

Examples are: XMLValidator and XMLPacker objects, which provide SciELO PS
validation behaviour and packaging functionality, respectively.
"""
from __future__ import unicode_literals
import logging
from copy import deepcopy
try:
    import reprlib
except ImportError:
    import repr as reprlib

from lxml import etree

from . import utils, catalogs, style_errors, exceptions


__all__ = ['XMLValidator', 'HTMLGenerator']


LOGGER = logging.getLogger(__name__)


def _get_public_ids(sps_version):
    """Returns the set of allowed public ids for the XML based on its version.
    """
    if sps_version in ['pre-sps', 'sps-1.1']:
        return frozenset(catalogs.ALLOWED_PUBLIC_IDS_LEGACY)
    else:
        return frozenset(catalogs.ALLOWED_PUBLIC_IDS)


def _init_sps_version(xml_et, supported_versions=None):
    """Returns the SPS spec version for `xml_et` or raises ValueError.

    It also checks if the version is currently supported.

    :param xml_et: etree instance.
    :param supported_versions: (optional) the default value is set by env var `PACKTOOLS_SUPPORTED_SPS_VERSIONS`.
    """
    if supported_versions is None:
        supported_versions = catalogs.CURRENTLY_SUPPORTED_VERSIONS

    doc_root = xml_et.getroot()
    version_from_xml = doc_root.attrib.get('specific-use', None)
    if version_from_xml is None:
        raise exceptions.XMLSPSVersionError('cannot get the SPS version from /article/@specific-use')

    if version_from_xml not in supported_versions:
        raise exceptions.XMLSPSVersionError('version "%s" is not currently supported' % version_from_xml)
    else:
        return version_from_xml


def StdSchematron(schema_name):
    """Returns an instance of `isoschematron.Schematron`.

    A standard schematron is one bundled with packtools.
    The returned instance is cached due to performance reasons.

    :param schema_name: The logical name of schematron file in the package `catalog`.
    """
    cache = utils.setdefault(StdSchematron, 'cache', lambda: {})

    if schema_name in cache:
        return cache[schema_name]
    else:
        try:
            schema_path = catalogs.SCHEMAS[schema_name]
        except KeyError:
            raise ValueError('unrecognized schema: "%s"' % schema_name)

        schematron = utils.get_schematron_from_filepath(schema_path)
        cache[schema_name] = schematron
        return schematron


def XSLT(xslt_name):
    """Returns an instance of `etree.XSLT`.

    The returned instance is cached due to performance reasons.
    """
    cache = utils.setdefault(XSLT, 'cache', lambda: {})

    if xslt_name in cache:
        return cache[xslt_name]
    else:
        try:
            xslt_doc = etree.parse(catalogs.HTML_GEN_XSLTS[xslt_name])
        except KeyError:
            raise ValueError('unrecognized xslt: "%s"' % xslt_name)

        xslt = etree.XSLT(xslt_doc)
        cache[xslt_name] = xslt
        return xslt


#----------------------------------
# validators for etree._ElementTree
#
# a validator is an object that
# provides the method ``validate``,
# with the following signature:
# validate(xmlfile: etree._ElementTree) -> Tuple(bool, list)
#----------------------------------
class PyValidator(object):
    """Style validations implemented in Python.
    """
    def __init__(self, pipeline=catalogs.StyleCheckingPipeline, label=u''):
        self.ppl = pipeline()
        self.label = label

    def validate(self, xmlfile):
        errors = next(self.ppl.run(xmlfile, rewrap=True))
        for error in errors:
            error.label = self.label

        return bool(errors), errors


class DTDValidator(object):
    """DTD validations.
    """
    def __init__(self, dtd, label=u''):
        self.dtd = dtd
        self.label = label

    def validate(self, xmlfile):
        """Validate xmlfile against the given DTD.

        Returns a tuple comprising the validation status and the errors list.
        """
        result = self.dtd.validate(xmlfile)
        errors = [style_errors.SchemaStyleError(err, label=self.label)
                  for err in self.dtd.error_log]

        return result, errors


class SchematronValidator(object):
    """Style validations implemented in Schematron.
    """
    def __init__(self, sch, label=u''):
        self.sch = sch
        self.label = label

    @classmethod
    def from_catalog(cls, ref, **kwargs):
        """Get an instance based on schema's reference name.

        :param ref: The reference name for the schematron file in
                    :data:`packtools.catalogs.SCH_SCHEMAS`.
        """
        return cls(StdSchematron(ref), **kwargs)

    def validate(self, xmlfile):
        """Validate xmlfile against the given Schematron schema.

        Returns a tuple comprising the validation status and the errors list.
        """
        result = self.sch.validate(xmlfile)
        errors = [style_errors.SchematronStyleError(err, label=self.label)
                  for err in self.sch.error_log]

        return result, errors


def iter_schematronvalidators(iterable):
    """Returns a generator of :class:`packtools.domain.SchematronValidator`.

    :param iterable: an iterable where each item follows one of the forms
                     ``Iterable[isoschematron.Schematron]`` or
                     ``Iterable[Tuple[isoschematron.Schematron, str]]``. The
                     latter sets the label attribute of the validator instance.
    """
    for item in iterable:
        try:
            sch_obj, sch_label = item
        except TypeError:
            sch_obj = item
            sch_label = u''

        validator = SchematronValidator(sch_obj, label=sch_label)
        yield validator


#--------------------------------
# adapters for etree._ElementTree
#--------------------------------
class XMLValidator(object):
    """Adapter that performs SPS validations.

    SPS validation stages are:
      - JATS 1.0 or PMC 3.0 (as bound by the doctype declaration or passed
        explicitly)
      - SciELO Style - ISO Schematron
      - SciELO Style - Python based pipeline

    :param file: etree._ElementTree instance.
    :param sps_version: the version of the SPS that will be the basis for validation.
    :param dtd: (optional) etree.DTD instance. If not provided, we try the external DTD.
    :param style_validators: (optional) list of
                             :class:`packtools.domain.SchematronValidator`
                             objects.
    """
    def __init__(self, file, dtd=None, style_validators=None):
        assert isinstance(file, etree._ElementTree)

        self.lxml = file
        self.doctype = self.lxml.docinfo.doctype

        self.dtd = dtd or self.lxml.docinfo.externalDTD
        self.source_url = self.lxml.docinfo.URL
        self.public_id = self.lxml.docinfo.public_id
        self.encoding = self.lxml.docinfo.encoding

        if style_validators:
            self.style_validators = list(style_validators)
        else:
            self.style_validators = []

    @classmethod
    def parse(cls, file, no_doctype=False, sps_version=None,
            supported_sps_versions=None, extra_sch_schemas=None, **kwargs):
        """Factory of XMLValidator instances.

        If `file` is not an etree instance, it will be parsed using
        :func:`packtools.utils.XML`.

        If the DOCTYPE is declared, its public id is validated against a white list,
        declared by :data:`ALLOWED_PUBLIC_IDS` module variable. The system id is ignored.
        By default, the allowed values are:

          - SciELO PS >= 1.2:
            - ``-//NLM//DTD JATS (Z39.96) Journal Publishing DTD v1.0 20120330//EN``
          - SciELO PS 1.1:
            - ``-//NLM//DTD JATS (Z39.96) Journal Publishing DTD v1.0 20120330//EN``
            - ``-//NLM//DTD Journal Publishing DTD v3.0 20080202//EN``

        :param file: Path to the XML file, URL, etree or file-object.
        :param no_doctype: (optional) if missing DOCTYPE declaration is accepted.
        :param sps_version: (optional) force the style validation against a SPS version.
        :param supported_sps_versions: (optional) list of supported versions. the
               only way to bypass this restriction is by using the arg `sps_version`.
        :param extra_sch_schemas: (optional) list of extra Schematron schemas.
        """
        try:
            et = utils.XML(file)
        except TypeError:
            # We hope it is an instance of etree.ElementTree. If it is not,
            # it will fail in the next lines.
            et = file

        # can raise exception
        sps_version = sps_version or _init_sps_version(et, supported_sps_versions)

        # get the right Schematron validator based on the value of ``sps_version``
        # and then mix it with the list of schemas supplied by the user.
        LOGGER.info('auto-loading style validations for version "%s"', sps_version)

        auto_loaded_sch_label = u'@' + sps_version
        style_validators = [
                SchematronValidator.from_catalog(sps_version,
                    label=auto_loaded_sch_label),
                PyValidator(label=auto_loaded_sch_label),  # the python based validation pipeline
        ]
        if extra_sch_schemas:
            style_validators += list(
                    iter_schematronvalidators(extra_sch_schemas))

        allowed_public_ids = _get_public_ids(sps_version)

        # DOCTYPE declaration must be present by default. This behaviour can
        # be changed by the `no_doctype` arg.
        LOGGER.info('fetching the DOCTYPE declaration')
        doctype = et.docinfo.doctype
        if not doctype and not no_doctype:
            raise exceptions.XMLDoctypeError(
                    'cannot get the DOCTYPE declaration')

        # if there exists a DOCTYPE declaration, ensure its PUBLIC-ID is
        # supported.
        LOGGER.info('fetching the PUBLIC-ID in DOCTYPE declaration')
        public_id = et.docinfo.public_id
        if doctype and public_id not in allowed_public_ids:
            raise exceptions.XMLDoctypeError('invalid DOCTYPE public id')

        return cls(et, style_validators=style_validators, **kwargs)

    @property
    def sps_version(self):
        doc_root = self.lxml.getroot()
        sps_version = doc_root.attrib.get('specific-use', None)
        return sps_version

    @property
    def dtd_validator(self):
        if self.dtd:
            return DTDValidator(self.dtd)
        else:
            return None

    @utils.cachedmethod
    def validate(self):
        """Validate the source XML against JATS DTD.

        Returns a tuple comprising the validation status and the errors list.
        """
        if self.dtd_validator is None:
            raise exceptions.UndefinedDTDError('cannot validate (DTD is not set)')

        result_tuple = self.dtd_validator.validate(self.lxml)
        return result_tuple

    @utils.cachedmethod
    def validate_style(self):
        """Validate the source XML against SPS-Style Tagging guidelines.

        Returns a tuple comprising the validation status and the errors list.
        """
        errors = []
        for validator in self.style_validators:
            LOGGER.info('running validator "%s"', repr(validator))
            errors += validator.validate(self.lxml)[1]

        result = not bool(errors)

        return result, errors

    def validate_all(self, fail_fast=False):
        """Runs all validations.

        First, the XML is validated against the DTD (calling :meth:`validate`).
        If no DTD is provided and the argument ``fail_fast == True``, a ``TypeError``
        is raised. After that, the XML is validated against the SciELO style
        (calling :meth:`validate_style`).

        :param fail_fast: (optional) raise ``TypeError`` if the DTD has not been loaded.
        """
        try:
            v_result, v_errors = self.validate()

        except exceptions.UndefinedDTDError:
            if fail_fast:
                raise
            else:
                v_result = None
                v_errors = []

        s_result, s_errors = self.validate_style()

        val_status = False if s_result is False else v_result
        val_errors = v_errors + s_errors

        return val_status, val_errors

    def _annotate_error(self, element, error):
        """Add an annotation prior to `element`, with `error` as the content.

        The annotation is a comment added prior to `element`.

        :param element: etree instance to be annotated.
        :param error: string of the error.
        """
        notice_element = etree.Element('SPS-ERROR')
        notice_element.text = error
        element.addprevious(etree.Comment('SPS-ERROR: %s' % error))

    def annotate_errors(self, fail_fast=False):
        """Add notes on all elements that have errors.

        The errors list is generated as the result of calling :meth:`validate_all`.
        """
        status, errors = self.validate_all(fail_fast=fail_fast)
        mutating_xml = deepcopy(self.lxml)

        if status is True:
            return mutating_xml

        err_pairs = []
        for error in errors:
            try:
                err_element = error.get_apparent_element(mutating_xml)
            except ValueError:
                err_element = mutating_xml.getroot()

            err_pairs.append((err_element, error.message))

        for el, em in err_pairs:
            self._annotate_error(el, em)

        return mutating_xml

    def __repr__(self):
        arg_names = [u'lxml', u'sps_version', u'dtd']
        arg_values = [reprlib.repr(getattr(self, arg)) for arg in arg_names]
        arg_names[0] = u'file'

        args = zip(arg_names, arg_values)
        attrib_args = (u'{}={}'.format(name, value) for name, value in args)

        return '<XMLValidator object at 0x%x (%s)>' % (
                id(self), u', '.join(attrib_args))

    @property
    def meta(self):
        """Article metadata.
        """
        parsed_xml = self.lxml

        xml_nodes = {
            "journal_title": "front/journal-meta/journal-title-group/journal-title",
            "journal_eissn": "front/journal-meta/issn[@pub-type='epub']",
            "journal_pissn": "front/journal-meta/issn[@pub-type='ppub']",
            "article_title": "front/article-meta/title-group/article-title",
            "issue_year": "front/article-meta/pub-date/year",
            "issue_volume": "front/article-meta/volume",
            "issue_number": "front/article-meta/issue",
        }

        metadata = {'filename': self.source_url}
        for node_k, node_v in xml_nodes.items():
            node = parsed_xml.find(node_v)
            metadata[node_k] = getattr(node, 'text', None)

        return metadata

    @property
    def assets(self):
        """Lists all static assets referenced by the XML.
        """
        return utils.get_static_assets(self.lxml)

    def lookup_assets(self, base):
        """Look for each asset in `base`, and returns a list of tuples
        with the asset name and its presence status.

        :param base: any container that implements membership tests, i.e.
                     it must support the ``in`` operator.
        """
        return [(asset, asset in base) for asset in self.assets]


class HTMLGenerator(object):
    """Adapter that generates HTML from SPS XML.

    Basic usage:

    .. code-block:: python

        from lxml import etree

        xml = etree.parse('valid-sps-file.xml')
        generator = HTMLGenerator(xml)

        html = generator.generate('pt')
        html_string = etree.tostring(html, encoding='unicode', method='html')


    :param file: etree._ElementTree instance.
    :param xslt: (optional) etree.XSLT instance. If not provided, the default XSLT is used.
    :param css: (optional) URI for a CSS file.
    """
    def __init__(self, file, xslt=None, css=None, print_css=None, js=None, permlink=None, url_article_page=None, url_download_ris=None):
        assert isinstance(file, etree._ElementTree)

        self.lxml = file
        self.xslt = xslt or XSLT('root-html-2.0.xslt')
        self.css = css
        self.print_css = print_css
        self.js = js
        self.permlink = permlink
        self.url_article_page = url_article_page
        self.url_download_ris = url_download_ris

    @classmethod
    def parse(cls, file, valid_only=True, **kwargs):
        """Factory of HTMLGenerator instances.

        If `file` is not an etree instance, it will be parsed using
        :func:`XML`.

        :param file: Path to the XML file, URL, etree or file-object.
        :param valid_only: (optional) prevents the generation of HTML for invalid XMLs.
        """
        if isinstance(file, etree._ElementTree):
            et = file
        else:
            et = utils.XML(file)

        if valid_only:
            is_valid, _ = XMLValidator.parse(et).validate_all()
            if not is_valid:
                raise ValueError('invalid XML')

        return cls(et, **kwargs)

    @property
    def languages(self):
        """The language of the main document plus all translations.
        """
        return self.lxml.xpath(
            '/article/@xml:lang | //sub-article[@article-type="translation"]/@xml:lang')

    @property
    def language(self):
        """The language of the main document.
        """
        try:
            return self.lxml.xpath('/article/@xml:lang')[0]
        except IndexError:
            return None

    def _is_aop(self):
        """ Has the document been published ahead-of-print?
        """
        volume = self.lxml.findtext('front/article-meta/volume')
        number = self.lxml.findtext('front/article-meta/issue')

        return volume == '00' and number == '00'

    def _get_issue_label(self):
        volume = self.lxml.findtext('front/article-meta/volume')
        number = self.lxml.findtext('front/article-meta/issue')

        return 'vol.%s n.%s' % (volume, number)

    def _get_bibliographic_legend(self):
        return '[#BIBLIOGRAPHIC LEGEND#]'

        issue = 'ahead of print' if self._is_aop() else self._get_issue_label()

        abrev_title = self.lxml.findtext(
            'front/journal-meta/journal-title-group/abbrev-journal-title[@abbrev-type="publisher"]')
        city = '[#CITY#]'

        pubdate = self.lxml.xpath(
            '/article/front/article-meta/pub-date[@pub-type="epub-ppub" or @pub-type="epub"][1]')[0]
        pubtype = 'Epub' if pubdate.xpath('@pub-type')[0] == 'epub' else ''
        day = pubdate.findtext('day')
        month = pubdate.findtext('month')
        year = pubdate.findtext('year')
        dates = ' '.join([month, year]) if month else year

        parts = [abrev_title, issue, city, pubtype, dates]

        return ' '.join([part for part in parts if part])

    def __iter__(self):
        """Iterates thru all languages and generates the HTML for each one.
        """
        for lang in self.languages:
            res_html = self.generate(lang)
            yield lang, res_html

    def generate(self, lang):
        """Generates the HTML in the language ``lang``.

        :param lang: 2-digit ISO 639-1 text string.
        """
        main_language = self.language
        if main_language is None:
            raise exceptions.HTMLGenerationError('main document language is '
                                                 'undefined.')

        if lang not in self.languages:
            raise ValueError('unrecognized language: "%s"' % lang)

        is_translation = lang != main_language
        return self.xslt(
                self.lxml,
                article_lang=etree.XSLT.strparam(lang),
                is_translation=etree.XSLT.strparam(str(is_translation)),
                bibliographic_legend=etree.XSLT.strparam(self._get_bibliographic_legend()),
                issue_label=etree.XSLT.strparam(self._get_issue_label()),
                styles_css_path=etree.XSLT.strparam(self.css or ''),
                print_styles_css_path=etree.XSLT.strparam(self.print_css or ''),
                js_path=etree.XSLT.strparam(self.js or ''),
                permlink=etree.XSLT.strparam(self.permlink or ''),
                url_article_page=etree.XSLT.strparam(self.url_article_page or ''),
                url_download_ris=etree.XSLT.strparam(self.url_download_ris or ''),
        )



"""Just to ease the access to the files.
"""
import os
import sys

from . import checks 


_CWD = os.path.dirname(os.path.abspath(__file__))


default_catalog = {
        'NAME': 'SciELO Style Catalog for Packtools',

        # Validation schemas: XSD or DTD files.
        'SCH_SCHEMAS': {
            'sps-1.1': os.path.join(_CWD, 'scielo-style-1.1.sch'),
            'sps-1.2': os.path.join(_CWD, 'scielo-style-1.2.sch'),
            'sps-1.3': os.path.join(_CWD, 'scielo-style-1.3.sch'),
            'sps-1.4': os.path.join(_CWD, 'scielo-style-1.4.sch'),
            'sps-1.5': os.path.join(_CWD, 'scielo-style-1.5.sch'),
            'sps-1.6': os.path.join(_CWD, 'scielo-style-1.6.sch'),
            'sps-1.7': os.path.join(_CWD, 'scielo-style-1.7.sch'),
            'sps-1.8': os.path.join(_CWD, 'scielo-style-1.8.sch'),
            'sps-1.9': os.path.join(_CWD, 'scielo-style-1.9.sch'),
            'sps-1.10': os.path.join(_CWD, 'scielo-style-1.10.sch'),

            # Collection-specific schema
            'scielo-br': os.path.join(_CWD, 'scielo-br.sch'),
        },
        'DTDS': {
            'JATS-journalpublishing1.dtd': os.path.join(
                _CWD, 'jats-publishing-dtd-1.0/JATS-journalpublishing1.dtd'),
            'journalpublishing3.dtd': os.path.join(
                _CWD, 'pmc-publishing-dtd-3.0/journalpublishing3.dtd'),
        },

        # XML Catalog - OASIS Standard.
        'XML_CATALOG': os.path.join(_CWD, 'scielo-publishing-schema.xml'),

        # HTML GENERATOR default constants
        'HTML_GEN_XSLTS': {
            'root-html-1.2.xslt': os.path.join(_CWD, 'htmlgenerator/root-html-1.2.xslt'),
            'root-html-2.0.xslt': os.path.join(_CWD, 'htmlgenerator/root-html-2.0.xslt'),
        },
        'HTML_GEN_DEFAULT_PRINT_CSS_PATH': os.path.join(_CWD,
            'htmlgenerator/static/scielo-bundle-print.css'),
        'HTML_GEN_DEFAULT_CSS_PATH': os.path.join(_CWD,
            'htmlgenerator/static/scielo-article-standalone.css'),
        'HTML_GEN_DEFAULT_JS_PATH': os.path.join(_CWD,
            'htmlgenerator/static/scielo-article-standalone-min.js'),

        # As a general rule, only the latest 2 versions are supported simultaneously.
        'CURRENTLY_SUPPORTED_VERSIONS': os.environ.get(
            'PACKTOOLS_SUPPORTED_SPS_VERSIONS', 'sps-1.8:sps-1.9').split(':'),

        'ALLOWED_PUBLIC_IDS': (
            '-//NLM//DTD JATS (Z39.96) Journal Publishing DTD v1.0 20120330//EN',
            '-//NLM//DTD JATS (Z39.96) Journal Publishing DTD v1.1 20151215//EN',
        ),
        # doctype public ids for sps <= 1.1
        'ALLOWED_PUBLIC_IDS_LEGACY': (
            '-//NLM//DTD JATS (Z39.96) Journal Publishing DTD v1.0 20120330//EN',
            '-//NLM//DTD Journal Publishing DTD v3.0 20080202//EN',
        ),
}

# Python>=3.5 is possible to use the syntax: SCHEMAS = {**SCH_SCHEMAS, **DTDS}
# https://docs.python.org/dev/whatsnew/3.5.html#pep-448-additional-unpacking-generalizations
default_catalog['SCHEMAS'] = dict(default_catalog['SCH_SCHEMAS'])
default_catalog['SCHEMAS'].update(default_catalog['DTDS'])


class Catalog(object):
    """A catalog is a namespace that groups constants together and makes them
    accessible through *dot* accessors.
    """
    ISO3166_CODES = os.path.join(_CWD, 'iso3166-codes.json')

    def __init__(self, **kwargs):
        for name, value in kwargs.items():
            setattr(self, name, value)

    @classmethod
    def fromdict(cls, d):
        return cls(**d)


class CatalogLoader(object):
    group_name = 'packtools.catalog'

    def _load_plugin_if_exists(self, name):
        """Returns the plugged-in Catalog if it exists or None.
        """
        from pkg_resources import iter_entry_points
        for entry_point in iter_entry_points(group=self.group_name, name=None):
            if entry_point.name == name:
                return entry_point.load()

        return None

    def load(self, name, default, plugin_wrapper=None):
        """Returns the plugged-in Catalog if it exists or otherwise the default
        catalog.

        :param default: object that will be returned if ``name`` is not set.
        :param plugin_wrapper: a callable that will be called with the loaded
        object as argument.
        """
        plugin_wrapper = plugin_wrapper if plugin_wrapper else lambda p: p

        plugin = self._load_plugin_if_exists(name) or default
        return plugin_wrapper(plugin)


class PluggableModule(object):
    def __init__(self):
        self.catalog = CatalogLoader().load('packtools_catalog',
                default=default_catalog,
                plugin_wrapper=lambda p: Catalog.fromdict(p)) 
        self.StyleCheckingPipeline = CatalogLoader().load('packtools_checks',
                default=checks.StyleCheckingPipeline)
        self.checks = checks

    def __getattr__(self, name):
        try:
            return getattr(self.catalog, name)
        except AttributeError:
            return getattr(self.StyleCheckingPipeline, name)

# This is a documented and recommended hack. See:
# https://mail.python.org/pipermail/python-ideas/2012-May/014969.html
sys.modules[__name__] = PluggableModule()



# coding: utf-8
import os
import pkg_resources
import argparse
import logging
import sys

import packtools


LOGGER = logging.getLogger(__name__)


@packtools.utils.config_xml_catalog
def main():

    packtools_version = pkg_resources.get_distribution("packtools").version

    parser = argparse.ArgumentParser(description="WEB Images generator CLI utility")
    parser.add_argument("SPPackage", help="SP Package Zip file path.")
    parser.add_argument(
        "New_SPPackage", nargs="?", default='', help="Optimised SP Package Zip file path."
    )
    parser.add_argument(
        '--preservefiles',
        action='store_true',
        help='preserve extracted and optimised files in aux directory',
    )
    parser.add_argument(
        '--stopiferror',
        action='store_true',
        help='stop execution if an error occurs',
    )
    parser.add_argument("--version", action="version", version=packtools_version)
    parser.add_argument("--loglevel", default="WARNING")
    args = parser.parse_args()

    logging.basicConfig(level=getattr(logging, args.loglevel.upper()))

    print("Please wait, this may take a while...", file=sys.stderr)

    if len(args.New_SPPackage) > 0:
        new_package_file_path = args.New_SPPackage
    else:
        new_package_file_path = os.path.splitext(args.SPPackage)[0] + "_optimised.zip"

    package = packtools.SPPackage.from_file(
        args.SPPackage,
        os.path.splitext(args.SPPackage)[0],
        stop_if_error=args.stopiferror,
    )
    package.optimise(
        new_package_file_path=new_package_file_path, preserve_files=args.preservefiles
    )


if __name__ == "__main__":
    main()



#coding: utf-8
from __future__ import unicode_literals
import logging
import itertools
import json

import plumber

from packtools.style_errors import StyleError

LOGGER = logging.getLogger(__name__)


def ISO3166_CODES_SET():
    from . import ISO3166_CODES
    with open(ISO3166_CODES) as f:
        return set(json.load(f))


# --------------------------------
# Basic functionality
# --------------------------------
@plumber.filter
def setup(message):
    """Prepare the message to traverse the pipeline.

    The input `message` is an `etree` instance. The pipeline will inspect
    this etree and append the errors on an errors list. This errors list
    is instantiated at this setup pipe.
    """
    return message, []


@plumber.filter
def teardown(message):
    """Finalize the processing pipeline and return the errors list.
    """
    _, err_list = message
    return err_list


def StyleCheckingPipeline():
    """Factory for style checking pipelines.
    """
    return plumber.Pipeline(setup, funding_group, doctype, country_code, teardown)


# --------------------------------
# Funding Group check
# --------------------------------
@plumber.filter
def funding_group(message):
    """Validate the Funding Group element

    Ref. URI:
      - http://jats.nlm.nih.gov/publishing/tag-library/1.1d1/n-pdx0.html
      - http://ref.scielo.org/g388x3

    Element may be found in:
      - /article/front/article-meta/funding_group
    """
    et, err_list = message

    funding_groups = et.findall('front//funding-group//award-id')
    financial_disclosures = et.findall(
            'back//fn[@fn-type="financial-disclosure"]')
    has_funding_group = bool(all([bool(elem.text)
                                  for elem in funding_groups]))
    has_financial_disclosure = bool(financial_disclosures)

    if has_financial_disclosure and not has_funding_group or (
            len(financial_disclosures) > len(funding_groups)):
        err = StyleError()
        err.message = "Element 'fn-group': This element has occurrences not declared in funding-group."
        err_list.append(err)

    if has_funding_group:
        def get_text(elem):
            """Always returns a text string.
            """
            try:
                return u''.join(elem.itertext())
            except:
                return u''

        # only the main document is relevant
        award_ids = [elem.text for elem in et.findall(
            'front//funding-group/award-group/award-id')
            if elem.text is not None]
        fn_occs = [get_text(elem) for elem in et.findall(
            'back//fn[@fn-type="financial-disclosure"]/p')]
        ack_occs = [get_text(elem) for elem in et.findall(
            'back//ack/p')]

        def in_there(award_id, texts):
            for text in texts:
                if award_id in text:
                    return True
                else:
                    LOGGER.info('cannot find award-id "%s" in text', award_id)

            return False

        LOGGER.info('declared contract numbers: %s', award_ids)

        missing_award_ids = set(award_ids)
        paragraphs = [p for p in itertools.chain(fn_occs, ack_occs) if p]

        for award_id in award_ids:
            if in_there(award_id, paragraphs):
                try:
                    missing_award_ids.remove(award_id)
                except KeyError:
                    LOGGER.info('many occurences of award-id: "%s"',
                                award_id)
            else:
                LOGGER.info('cannot find contract number "%s" in set "%s"',
                        award_id, award_ids)

        if missing_award_ids:
            LOGGER.info('missing award-id: "%s"', missing_award_ids)
            err = StyleError()
            err.message = "Element 'funding-group': This element has occurrences not declared in fn or ack."
            err_list.append(err)
    else:
        LOGGER.info('no contract numbers found in %s', et)

    return message


@plumber.filter
def doctype(message):
    """Make sure the DOCTYPE declaration is present.
    """
    et, err_list = message

    if not et.docinfo.doctype:
        err = StyleError()
        err.message = "Missing DOCTYPE declaration."
        err_list.append(err)

    return message


@plumber.filter
def country_code(message):
    """Check country codes against iso3166 alpha-2 list.
    """
    et, err_list = message

    elements = et.findall('//*[@country]')
    for elem in elements:
        value = elem.attrib['country']
        if value not in ISO3166_CODES_SET():
            err = StyleError()
            err.line = elem.sourceline
            err.message = "Element '%s', attribute country: Invalid country code \"%s\"." % (elem.tag, value)
            err_list.append(err)

    return message




class PacktoolsError(Exception):
    """ The root of all evil in this lib.
    """


class XMLDoctypeError(PacktoolsError, TypeError):
    """ To handle missing or unsupported DOCTYPE definitions.
    """


class XMLSPSVersionError(PacktoolsError, ValueError):
    """ To handle missing or unsupported SPS versions.
    """


class UndefinedDTDError(PacktoolsError, TypeError):
    """ To handle validation attempts without a previously defined validator.
    """


class HTMLGenerationError(PacktoolsError):
    """ Generic errors during HTML generation phase.
    """


class SPPackageError(PacktoolsError):
    """ Generic errors during SP Package optimisation.
    """


class XMLWebOptimiserError(SPPackageError):
    """ Generic errors during XML WEB optimisation.
    """


class WebImageGeneratorError(SPPackageError):
    """ Generic errors during WEB Image optimisation.
    """



# coding:utf-8
import os
import logging

from flask import Flask, render_template
from flask_babelex import Babel

from .custom_filters import clean_uri, utility_processor
from .views import main as main_bp


LOGGER = logging.getLogger(__name__)


def create_app(settings_ns):

    babel = Babel()
    app = Flask(
        __name__,
        static_url_path="/static",
        static_folder="static",
        instance_relative_config=False,
    )

    app.config.from_object(settings_ns)

    app.register_blueprint(main_bp)
    app.jinja_env.filters["clean_uri"] = clean_uri
    app.context_processor(utility_processor)
    babel.init_app(app)

    if app.config["DEBUG"]:
        try:
            from flask_debugtoolbar import DebugToolbarExtension
        except ImportError:
            LOGGER.info('cannot import lib "flask_debugtoolbar". '
                        'Make sure it is installed and available in the '
                        'import path.')
        else:
            toolbar = DebugToolbarExtension()
            toolbar.init_app(app)

    return app


app = create_app(os.environ.get(
    "APP_SETTINGS", "packtools.webapp.config.default.ProductionConfig"))



# coding: utf-8
import lxml
from flask import current_app

import packtools


def count(target, collection, key):
    """Total target count on collection.

    :param key: callable to get the comparison value
    """
    occurences = sum([1 for item in collection if key(item) == key(target)])
    return occurences


def make_error_filter(key):
    """Filtering function factory

    :param key: callable to get the filtering value
    """
    known_errors = set()

    def err_filter(err):
        _err = key(err)

        is_known = _err in known_errors
        if is_known is False:
            known_errors.add(_err)

        return not is_known

    return err_filter


def analyze_xml(file, extra_schematron=None):
    """Analyzes `file` against packtools' XMLValidator.
    """
    result = err = None
    if extra_schematron:
        extra_sch = packtools.utils.get_schematron_from_filepath(extra_schematron)
        extra_sch = [extra_sch]
    else:
        extra_sch = None

    try:
        xml = packtools.XMLValidator.parse(file, extra_sch_schemas=extra_sch)

    except (
        lxml.etree.XMLSyntaxError,
        IOError,
        ValueError,
        packtools.exceptions.XMLDoctypeError,
    ) as e:
        err = e

    else:
        status, errors = xml.validate_all()
        err_xml = lxml.etree.tostring(
            xml.annotate_errors(),
            pretty_print=True,
            encoding="utf-8",
            xml_declaration=True,
        )

        result = {
            "annotations": err_xml.decode("utf-8"),
            "validation_errors": None,
            "meta": xml.meta,
            "sps_version": xml.sps_version,
            "is_deprecated_version": xml.sps_version
            == current_app.config.get("PACKTOOLS_DEPRECATION_WARNING_VERSION"),
        }

        if not status:
            err_filter = make_error_filter(lambda x: x.message)
            unique_err_list = filter(err_filter, errors)
            err_list = [
                (error, count(error, errors, lambda x: x.message))
                for error in unique_err_list
            ]

            result["validation_errors"] = err_list

    return result, err



"""Packtools is a Python library and set of command line utilities which can be
used to handle SciELO Publishing Schema packages and XML files.
"""
import os
import sys
import logging
import platform

from lxml import etree

from .domain import XMLValidator, HTMLGenerator
from .utils import XML, SPPackage, XMLWebOptimiser
from .version import __version__


__all__ = [
    'XMLValidator',
    'XML',
    '__version__',
    'get_debug_info',
    'HTMLGenerator',
    'SPPackage',
    'XMLWebOptimiser',
]


# we need to handle this exception in order to use sphinx-doc's autodoc.
try:
    LIBXML_COMPILED_VERSION = '.'.join(
            (str(digit) for digit in etree.LIBXML_COMPILED_VERSION))
except TypeError:
    LIBXML_COMPILED_VERSION = ''

try:
    LIBXML_VERSION = '.'.join(
            (str(digit) for digit in etree.LIBXML_VERSION))
except TypeError:
    LIBXML_VERSION = ''

try:
    LIBXSLT_COMPILED_VERSION = '.'.join(
            (str(digit) for digit in etree.LIBXSLT_COMPILED_VERSION))
except TypeError:
    LIBXSLT_COMPILED_VERSION = ''

try:
    LIBXSLT_VERSION = '.'.join(
            (str(digit) for digit in etree.LIBXSLT_VERSION))
except TypeError:
    LIBXSLT_VERSION = ''

try:
    LXML_VERSION = '.'.join(
            (str(digit) for digit in etree.LXML_VERSION))
except TypeError:
    LXML_VERSION = ''


def get_debug_info():
    """ Returns debug data computed at the time.
    """
    info = {
        'libxml_compiled_version': LIBXML_COMPILED_VERSION,
        'libxml_version': LIBXML_VERSION,
        'libxslt_compiled_version': LIBXSLT_COMPILED_VERSION,
        'libxslt_version': LIBXSLT_VERSION,
        'lxml_version': LXML_VERSION,
        'xml_catalog_files': os.environ.get('XML_CATALOG_FILES', ''),
        'system_path': sys.path,
        'packtools_version': __version__,
        'python_version': platform.python_version(),
    }

    return info


# Setting up a do-nothing handler. We expect the application to define
# the handler for `packtools`.
logging.getLogger(__name__).addHandler(logging.NullHandler())




# coding:utf-8
import os
import pkg_resources


try:
    PACKTOOLS_VERSION = pkg_resources.get_distribution("packtools").version
except pkg_resources.DistributionNotFound:
    PACKTOOLS_VERSION = None


class ProductionConfig(object):
    DEBUG = False
    TESTING = False
    CSRF_ENABLED = True

    # Linguagens suportados
    LANGUAGES = {"pt_BR": "Português", "en": "English", "es": "Español"}

    # linguagem padrão:
    BABEL_DEFAULT_LOCALE = "pt_BR"
    SECRET_KEY = os.environ.get("SECRET_KEY", "s3kr3tk3y")
    LOCAL_ZONE = os.environ.get("LOCAL_ZONE", "America/Sao_Paulo")
    SETTINGS_MAX_UPLOAD_SIZE = 512 * 1024  # max size in byte to upload xml to validator
    PACKTOOLS_VERSION = PACKTOOLS_VERSION

    # Versão mais antiga suportada do SPS, na qual o usuario do
    # StyleChecker deve ser notficado do Deprecation Warning.
    # A versão deve respeitar o formato do atributo @specific-use: 'sps-1.1' ou 'sps-1.2'.
    PACKTOOLS_DEPRECATION_WARNING_VERSION = "sps-1.1"


class DevelopmentConfig(ProductionConfig):
    DEVELOPMENT = True
    DEBUG = True

    DEBUG_TB_INTERCEPT_REDIRECTS = False
    DEBUG_TB_PANELS = [
        # default:
        "flask_debugtoolbar.panels.versions.VersionDebugPanel",
        "flask_debugtoolbar.panels.timer.TimerDebugPanel",
        "flask_debugtoolbar.panels.headers.HeaderDebugPanel",
        "flask_debugtoolbar.panels.request_vars.RequestVarsDebugPanel",
        "flask_debugtoolbar.panels.config_vars.ConfigVarsDebugPanel",
        "flask_debugtoolbar.panels.template.TemplateDebugPanel",
        "flask_debugtoolbar.panels.sqlalchemy.SQLAlchemyDebugPanel",
        "flask_debugtoolbar.panels.logger.LoggingPanel",
        "flask_debugtoolbar.panels.route_list.RouteListDebugPanel",
        "flask_debugtoolbar.panels.profiler.ProfilerDebugPanel",
    ]


class TestingConfig(ProductionConfig):
    TESTING = True



#coding: utf-8
from __future__ import unicode_literals
import logging
import functools
import itertools
import os
import glob
import sys
import json
import unicodedata
import zipfile
import io

from lxml import etree, isoschematron
from PIL import Image, ImageFile
try:
    import pygments     # NOQA
    from pygments.lexers import get_lexer_for_mimetype
    from pygments.formatters import TerminalFormatter
except ImportError:
    pygments = False    # NOQA

from packtools import catalogs, exceptions


LOGGER = logging.getLogger(__name__)


PY2 = sys.version_info[0] == 2


try:
    # available on lxml >= 3.4.0
    NOIDS_XMLPARSER = etree.XMLParser(collect_ids=False)
except TypeError:
    LOGGER.info('cannot instantiate an XML parser that avoids the collection '
                'of ids from elements.')
    NOIDS_XMLPARSER = etree.XMLParser()


def setdefault(object, attribute, producer):
    """
    Like dict().setdefault but for object attributes.
    """
    if not hasattr(object, attribute):
        setattr(object, attribute, producer())

    return getattr(object, attribute)


def cachedmethod(wrappee):
    """Caches method calls within known arguments.
    """
    @functools.wraps(wrappee)
    def wrapper(self, *args, **kwargs):
        key = (args, tuple(kwargs.items()))
        cache_attrname = '__' + wrappee.__name__

        cache = setdefault(self, cache_attrname, lambda: {})
        if key not in cache:
            cache[key] = wrappee(self, *args, **kwargs)

        return cache[key]

    return wrapper


def get_static_assets(xml_et):
    """Returns an iterable with all static assets referenced by xml_et.
    """
    paths = [
        '//graphic[@xlink:href]',
        '//media[@xlink:href]',
        '//inline-graphic[@xlink:href]',
        '//supplementary-material[@xlink:href]',
        '//inline-supplementary-material[@xlink:href]',
    ]

    iterators = [xml_et.iterfind(path, namespaces={'xlink': 'http://www.w3.org/1999/xlink'})
            for path in paths]

    elements = itertools.chain(*iterators)

    return [element.attrib['{http://www.w3.org/1999/xlink}href'] for element in elements]


def XML(file, no_network=True, load_dtd=True):
    """Parses `file` to produce an etree instance.

    The XML can be retrieved given its filesystem path,
    an URL or a file-object.

    :param file: Path to the XML file, URL or file-object.
    :param no_network: (optional) prevent network access for external DTD.
    :param load_dtd: (optional) load DTD during parse-time.
    """
    parser = etree.XMLParser(remove_blank_text=True,
                             load_dtd=load_dtd,
                             no_network=no_network)
    xml = etree.parse(file, parser)

    return xml


def get_schematron_from_buffer(buff, parser=NOIDS_XMLPARSER):
    """Returns an ``isoschematron.Schematron`` for ``buff``.

    The default parser doesn't collect ids on a hash table, i.e.:
    ``collect_ids=False``.
    """
    xmlschema_doc = etree.parse(buff, parser)
    return isoschematron.Schematron(xmlschema_doc)


def get_schematron_from_filepath(filepath):
    with open(filepath, mode='rb') as buff:
        return get_schematron_from_buffer(buff)


def config_xml_catalog(wrapped):
    """Decorator that wraps the execution of a function, setting-up and
    tearing-down the ``XML_CATALOG_FILES`` environment variable for the current
    process.

    .. code-block:: python

       @config_xml_catalog
       def main(xml_filepath):
           xml = XMLValidator(xml_filepath)
           # do some work here
    """
    @functools.wraps(wrapped)
    def wrapper(*args, **kwargs):
        try:
            os.environ['XML_CATALOG_FILES'] = catalogs.XML_CATALOG
            _return = wrapped(*args, **kwargs)

        finally:
            del(os.environ['XML_CATALOG_FILES'])

        return _return
    return wrapper


def flatten(paths):
    """ Produces absolute path for each path in paths.

    Glob expansions are allowed.

    :param paths: Collection of paths. A path can be relative, absolute or a 
                  glob expression.
    """
    def _inner_generator():
        for path in paths:
            ylock = True
            if not path.startswith(('http:', 'https:')):
                # try to expand wildchars and get the absolute path
                for fpath in glob.iglob(path):
                    yield os.path.abspath(fpath).strip()
                    ylock = False

            # args must not be suppressed, even the invalid
            if ylock:
                yield path.strip()

    for path in _inner_generator():
        if PY2:
            yield path.decode(encoding=sys.getfilesystemencoding())
        else:
            yield path


def prettify(jsonobj, colorize=True):
    """ Serialize and prettify a Python object as JSON.

    On windows, bypass pygments colorization.

    Function copied from Circus process manager:
    https://github.com/circus-tent/circus/blob/master/circus/circusctl.py
    """

    json_str = json.dumps(jsonobj, indent=2, sort_keys=True)
    if colorize and pygments and not sys.platform.startswith('win'):
        LOGGER.info('using pygments to highlight the output')
        try:
            lexer = get_lexer_for_mimetype("application/json")
            return pygments.highlight(json_str, lexer, TerminalFormatter())
        except Exception as exc:
            LOGGER.exception(exc)

    return json_str


def normalize_string(unistr):
    """Return the NFKC form for the unicode string ``unistr``.

    The normal form KD (NFKD) will apply the compatibility decomposition, i.e.
    replace all compatibility characters with their equivalents, followed by
    the canonical composition.
    """
    return unicodedata.normalize('NFKC', unistr)


class Xray(object):
    """Zip-file introspector.

    :param zip_file: instance of ``zipfile.ZipFile``.
    """
    def __init__(self, zip_file):
        self._zipfile = zip_file

    @classmethod
    def fromfile(cls, filepath):
        if not zipfile.is_zipfile(filepath):
            raise ValueError('cannot read "%s": not a valid zipfile' % filepath)

        zip_file = zipfile.ZipFile(filepath, 'r')
        return cls(zip_file)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def show_sorted_members(self):
        """Shows the package members sorted by their file extensions.
        """
        sorted_members = {}
        for member in self.show_members():
            _, ext = member.rsplit('.', 1)
            ext_node = sorted_members.setdefault(ext.lower(), [])
            ext_node.append(member)

        return sorted_members

    def show_members(self):
        """Shows the package members.
        """
        return [filename for fileinfo, filename
                in zip(self._zipfile.infolist(), self._zipfile.namelist())
                if fileinfo.file_size]

    def get_file(self, member, mode='r'):
        """Get file object for member.

        A complete list of members can be checked
        calling :meth:`show_members`.

        :param member: a zip member, e.g. 'foo.xml'
        """
        try:
            return self._zipfile.open(member, mode)

        except KeyError:
            raise ValueError('cannot open file "%s": file doesn\'t exist' % member)

    def close(self):
        """Close the archive file.
        """
        self._zipfile.close()


def resolve_schematron_filepath(value):
    """Determine the filepath for ``value``.

    The lookup is run against all known schemas from
    :data:`packtools.catalogs.SCH_SCHEMAS`. If ``value`` is already a filepath,
    than it is returned as it is.
    """
    try:
        lookup_builtin = value.startswith('@')
    except AttributeError as exc:
        # the `from` clause cannot be used due to compatibility with python 2.
        raise TypeError('invalid input type for text string: "value"')

    if lookup_builtin:
        path = catalogs.SCH_SCHEMAS.get(value[1:])
        if path:
            return path
        else:
            raise ValueError('cannot resolve schematron "%s"' % value)
    elif os.path.lexists(value):
        return value
    else:
        raise ValueError('could not locate file "%s" (I/O failure)' % value)


class WebImageGenerator:
    """Generate WEB Images versions of a given Image.

    Basic usage:

    .. code-block:: python

        image_generator = WebImageGenerator(image_filename, image_file_dir)
        new_filename = image_generator.convert2png()

    :param image_filename: image file name
    :param image_file_dir: directory where ``image_filename`` is and where new versions
        will be saved
    :param file_bytes: image file bytes
    """

    def __init__(self, image_filename, image_file_dir, file_bytes=None):
        self.filename = image_filename
        self.thumbnail_size = (267, 140)
        self.image_file_path = os.path.join(image_file_dir, image_filename)
        self._image_object = self._get_image_object(file_bytes)

    def _get_image_object(self, file_bytes):
        if file_bytes is not None:
            parser = ImageFile.Parser()
            try:
                parser.feed(file_bytes)
                image = parser.close()
            except (Image.DecompressionBombError, IOError) as exc:
                raise exceptions.WebImageGeneratorError(
                    'Error reading image "%s": %s' % (self.filename, str(exc))
                )
            else:
                return image

    @property
    def png_filename(self):
        return os.path.splitext(self.filename)[0] + ".png"

    @property
    def thumbnail_filename(self):
        return os.path.splitext(self.filename)[0] + ".thumbnail.jpg"

    def convert2png(self, destination_path=None):
        """Generate a PNG file from image file with the same name, changing only the
        file extension. If ``destination_path`` is given, the new image is saved in it,
        otherwise it is saved in the same directory as original image.
        """
        try:
            tiff_file = Image.open(self.image_file_path)
        except (Image.DecompressionBombError, OSError, IOError, ValueError) as exc:
            raise exceptions.WebImageGeneratorError(
                'Error opening image file "%s": %s' % (self.image_file_path, str(exc))
            )
        else:
            png_file = tiff_file.copy()
            new_filename = os.path.splitext(self.image_file_path)[0] + ".png"
            if destination_path is not None and len(destination_path) > 0:
                new_filename = os.path.join(
                    destination_path, os.path.basename(new_filename)
                )
            try:
                png_file.save(new_filename, "PNG")
                return new_filename
            except (ValueError, IOError) as exc:
                raise exceptions.WebImageGeneratorError(
                    'Error saving image file "%s": %s' % (new_filename, str(exc))
                )
            finally:
                tiff_file.close()

    def create_thumbnail(self, destination_path=None):
        """Generate a thumbnail file from image file with the same name, changing only
        the file name to ``*.thumbnail.jpg``. If ``destination_path`` is given, the new
        image is saved in it, otherwise it is saved in the same directory as original image.
        """
        try:
            image_file = Image.open(self.image_file_path)
        except (Image.DecompressionBombError, OSError, IOError, ValueError) as exc:
            raise exceptions.WebImageGeneratorError(
                'Error opening image file "%s": %s' % (self.image_file_path, str(exc))
            )
        else:
            thumbnail_file = image_file.copy()
            new_filename = os.path.splitext(self.image_file_path)[0] + ".thumbnail.jpg"
            if destination_path is not None and len(destination_path) > 0:
                new_filename = os.path.join(
                    destination_path, os.path.basename(new_filename)
                )
            thumbnail_file.thumbnail(self.thumbnail_size)
            try:
                thumbnail_file.save(new_filename, "JPEG")
                return new_filename
            except (ValueError, IOError) as exc:
                raise exceptions.WebImageGeneratorError(
                    'Error saving image file "%s": %s' % (new_filename, str(exc))
                )
            finally:
                image_file.close()

    def _get_bytes(self, format):
        image_file = io.BytesIO()
        try:
            self._image_object.convert("RGB").save(image_file, format)
        except (ValueError, IOError) as exc:
            raise exceptions.WebImageGeneratorError(
                'Error optimising image bytes from "%s": %s' % (self.filename, str(exc))
            )
        else:
            return image_file.getvalue()

    def get_png_bytes(self):
        """Generate a PNG image byte-like object from image file set in
        ``self._image_object``."""
        if self._image_object is None:
            raise exceptions.WebImageGeneratorError(
                'Error optimising image bytes from "%s": '
                "no original file bytes was given." % self.filename
            )

        return self._get_bytes("PNG")

    def get_thumbnail_bytes(self):
        """Generate a thumbnail image byte-like object from image file set in
        ``self._image_object``."""
        if self._image_object is None:
            raise exceptions.WebImageGeneratorError(
                'Error optimising image bytes from "%s": '
                'no original file bytes was given.'
                % self.filename
            )

        self._image_object.thumbnail(self.thumbnail_size)
        return self._get_bytes("JPEG")


class XMLWebOptimiser(object):
    """Optimise XML document to be properly rendered to HTML, with alternatives to
    images.

    Basic usage:

    .. code-block:: python

        xml_web_optimiser = XMLWebOptimiser(
            filename, image_filenames, read_file, stop_if_error
        )
        bytes = xml_web_optimiser.get_xml_file()
        optimised_assets = xml_web_optimiser.get_optimised_assets()
        assets_thumbnails = xml_web_optimiser.get_assets_thumbnails()

    :param filename: (str) XML file name
    :param image_filenames: (list) list of image file names 
    :param read_file: function to read file content from source which raises
        exceptions.SPPackageError if an error occurs during the file reading
    :param work_dir: directory path to work with image optimization
    :param stop_if_error: (bool) if True, it raises exceptions.XMLWebOptimiserError for
        handled exceptions, otherwise it logs error message.
    """

    def __init__(
        self, filename, image_filenames, read_file, work_dir, stop_if_error=False
    ):
        self.filename = filename
        self.work_dir = work_dir
        self.stop_if_error = stop_if_error
        self._optimised_assets = []
        self._assets_thumbnails = []
        if read_file is None:
            raise exceptions.XMLWebOptimiserError(
                "Error instantiating XMLWebOptimiser: read_file cannot be None"
            )
        self._read_file = read_file
        self._xml_file = XML(io.BytesIO(self._read_file(filename)), load_dtd=False)
        self._xml_doctype = self._xml_file.docinfo.doctype
        self._image_filenames = self._get_all_graphic_images_from_xml(image_filenames)

    def _get_all_graphic_images_from_xml(self, image_filenames):
        namespaces = {"xlink": "http://www.w3.org/1999/xlink"}
        graphic_filename = set()
        for elem in self._xml_file.xpath(
            './/graphic[@xlink:href] | .//inline-graphic[@xlink:href]',
            namespaces=namespaces
        ):
            href_text = elem.attrib.get("{http://www.w3.org/1999/xlink}href")
            if href_text is not None and href_text in image_filenames:
                graphic_filename.add(href_text)
            else:
                for image_filename in image_filenames:
                    if href_text in image_filename:
                        graphic_filename.add(image_filename)

        return graphic_filename

    def _handle_image_exception(self, exception):
        if self.stop_if_error:
            raise exception
        else:
            LOGGER.info(str(exception))

    def _get_all_images_to_optimise(self):
        def is_image_to_optimise(image):
            image_filename = image.attrib.get("{http://www.w3.org/1999/xlink}href", "")
            __, filename_ext = os.path.splitext(image_filename)
            if filename_ext.startswith(".tif") or len(filename_ext) == 0:
                is_optimised_siblings = [
                    sibling
                    for sibling in image.xpath(
                        '../{}[@specific-use="scielo-web"]'.format(image.tag),
                        namespaces={"xlink": "http://www.w3.org/1999/xlink"},
                    )
                    if sibling.tag == image.tag
                ]
                if len(is_optimised_siblings) == 0:
                    return True
            return False

        paths = [
            './/graphic[@xlink:href and not(@specific-use="scielo-web")]',
            './/inline-graphic[@xlink:href and not(@specific-use="scielo-web")]',
        ]
        namespaces = {"xlink": "http://www.w3.org/1999/xlink"}
        iterators = [
            self._xml_file.xpath(path, namespaces=namespaces) for path in paths
        ]
        for image in itertools.chain(*iterators):
            if is_image_to_optimise(image):
                image_filename = image.attrib["{http://www.w3.org/1999/xlink}href"]
                LOGGER.debug('Found <%s xlink:href="%s">', image.tag, image_filename)
                yield image_filename, image

    def _get_all_images_to_thumbnail(self):
        namespaces = {"xlink": "http://www.w3.org/1999/xlink"}
        images = self._xml_file.xpath("//graphic[@xlink:href]", namespaces=namespaces)
        images_parents = {image.getparent() for image in images}
        for images_parent in images_parents:
            alternatives = images_parent.xpath(
                "./graphic[@xlink:href]", namespaces=namespaces
            )
            thumbnail = images_parent.xpath(
                './graphic[@xlink:href and starts-with(@content-type, "scielo-")]',
                namespaces=namespaces,
            )
            if len(alternatives) == 1 or len(thumbnail) == 0:
                image_filename = alternatives[0].attrib.get(
                    "{http://www.w3.org/1999/xlink}href"
                )
                yield image_filename, alternatives[0]

    def _get_web_image_generator(self, image_filename):
        try:
            image_file_bytes = self._read_file(image_filename)
        except exceptions.SPPackageError as exc:
            self._handle_image_exception(exc)
        else:
            try:
                web_image_generator = WebImageGenerator(
                    image_filename, self.work_dir, image_file_bytes
                )
            except exceptions.WebImageGeneratorError as exc:
                self._handle_image_exception(exc)
            else:
                return web_image_generator

    def _add_optimised_image(self, image_filename):
        web_image_generator = self._get_web_image_generator(image_filename)
        if web_image_generator is not None:
            try:
                png_bytes = web_image_generator.get_png_bytes()
            except exceptions.WebImageGeneratorError as exc:
                self._handle_image_exception(exc)
            else:
                self._optimised_assets.append(
                    (web_image_generator.png_filename, png_bytes)
                )
                return web_image_generator.png_filename

    def _add_assets_thumbnails(self, image_filename):
        web_image_generator = self._get_web_image_generator(image_filename)
        if web_image_generator is not None:
            try:
                thumbnail_bytes = web_image_generator.get_thumbnail_bytes()
            except exceptions.WebImageGeneratorError as exc:
                self._handle_image_exception(exc)
            else:
                self._assets_thumbnails.append(
                    (web_image_generator.thumbnail_filename, thumbnail_bytes)
                )
                return web_image_generator.thumbnail_filename

    def _get_similar_filename(self, image_filename):
        for filename in self._image_filenames:
            filename_root, filename_ext = os.path.splitext(filename)
            if filename_root == image_filename and ".tif" in filename_ext:
                LOGGER.debug('Found similar file name: "%s"', image_filename)
                return True, filename
        else:
            msg_error = 'No file named "%s" in package'
            if self.stop_if_error:
                raise exceptions.XMLWebOptimiserError(msg_error % image_filename)
            else:
                LOGGER.error(msg_error, image_filename)
                return False, None

    def _get_optimised_image_with_filename(self, image_filename, add_image):
        optimise = True
        if image_filename not in self._image_filenames:
            LOGGER.debug('"%s" not found in package_files', image_filename)
            optimise, image_filename = self._get_similar_filename(image_filename)
        if optimise:
            LOGGER.debug('Optimising image file "%s"', image_filename)
            return add_image(image_filename)

    def _add_alternative_to_alternatives_tag(
        self, image_element, alternative_attr_values
    ):
        image_parent = image_element.getparent()
        new_alternative = etree.Element(image_element.tag)
        for attrb, value in alternative_attr_values:
            new_alternative.set(attrb, value)
        if image_parent.tag == "alternatives":
            image_parent.append(new_alternative)
        else:
            alternative_node = etree.Element("alternatives")
            alternative_node.tail = image_element.tail
            image_element.tail = None
            alternative_node.append(image_element)
            alternative_node.append(new_alternative)
            image_parent.append(alternative_node)

    def get_xml_file(self):
        """Get a byte-like optimised XML, with WEB alternatives for images."""
        for image_filename, image_element in self._get_all_images_to_optimise():
            new_filename = self._get_optimised_image_with_filename(
                image_filename, self._add_optimised_image
            )
            if new_filename is not None:
                alternative_attr_values = (
                    ("{http://www.w3.org/1999/xlink}href", new_filename),
                    ("specific-use", "scielo-web"),
                )
                self._add_alternative_to_alternatives_tag(
                    image_element, alternative_attr_values
                )

        for image_filename, image_element in self._get_all_images_to_thumbnail():
            new_filename = self._get_optimised_image_with_filename(
                image_filename, self._add_assets_thumbnails
            )
            if new_filename is not None:
                alternative_attr_values = (
                    ("{http://www.w3.org/1999/xlink}href", new_filename),
                    ("specific-use", "scielo-web"),
                    ("content-type", "scielo-267x140"),
                )
                self._add_alternative_to_alternatives_tag(
                    image_element, alternative_attr_values
                )

        return etree.tostring(
            self._xml_file,
            doctype=self._xml_doctype or None,
            xml_declaration=True,
            method="xml",
            encoding="utf-8",
            pretty_print=True,
        )

    def get_optimised_assets(self):
        """Generate tuples of PNG file name and bytes of each image produced by TIFF
        images referenced in XML content."""
        for optimised_asset in self._optimised_assets:
            yield optimised_asset

    def get_assets_thumbnails(self):
        """Generate tuples of thumbnail file name and bytes of each image produced by
        images referenced in XML content."""
        for asset_thumbnail in self._assets_thumbnails:
            yield asset_thumbnail


class SPPackage(object):
    """Adapter that manipulate SciELO Publishing Packages.

    Basic usage:

    .. code-block:: python

        package = SPPackage(package_file, extracted_package)
        package.optimise()

    :param package_file: SciELO Publishing Package, instance of ``zipfile.ZipFile``
    :param extracted_package: path to extract package files and optimise them
    """

    def __init__(self, package_file, extracted_package, stop_if_error=False):
        self._package_file = package_file
        self._extracted_package = extracted_package
        self._stop_if_error = stop_if_error

    @classmethod
    def from_file(cls, package_file_path, extracted_package=None, stop_if_error=False):
        """Factory of SPPackage instances.

        :param package_file_path: Path to the SciELO Publishing Package file, instance
               of ``zipfile.ZipFile``
        """

        if not zipfile.is_zipfile(package_file_path):
            raise ValueError("File is not a zip file")

        package2optimise = zipfile.ZipFile(package_file_path)
        if extracted_package is None:
            extracted_package = os.path.splitext(package_file_path)[0]
        return cls(package2optimise, extracted_package, stop_if_error)

    def _optimise_to_zipfile(
        self, new_package_file_path, xml_filename, zipped_filenames
    ):
        with zipfile.ZipFile(new_package_file_path, "a") as new_zip_file:
            zipped_files = []
            xml_web_optimiser = self._get_optimise_web_xml(
                xml_filename, zipped_filenames
            )
            # Write optimised XML to new Zipfile
            optimised_xml = xml_web_optimiser.get_xml_file()
            xml_zip_info = self._package_file.getinfo(xml_filename)
            LOGGER.debug('Writing XML file "%s" in package', xml_filename)
            new_zip_file.writestr(
                xml_filename, optimised_xml, xml_zip_info.compress_type
            )
            zipped_files.append(xml_filename)
            # Write optimised assets to new Zipfile
            LOGGER.debug('Writing asset files in package')
            for asset_filename, asset_bytes in xml_web_optimiser.get_optimised_assets():
                if asset_bytes is not None:
                    LOGGER.debug('Writing file "%s"', asset_filename)
                    new_zip_file.writestr(asset_filename, asset_bytes)
                    zipped_files.append(asset_filename)
            LOGGER.debug('Writing asset thumbnail files in package')
            for (
                asset_filename,
                asset_bytes,
            ) in xml_web_optimiser.get_assets_thumbnails():
                if asset_bytes is not None:
                    LOGGER.debug('Writing file "%s"', asset_filename)
                    new_zip_file.writestr(asset_filename, asset_bytes)
                    zipped_files.append(asset_filename)
            return zipped_files

    def _write_files_left(self, new_package_file_path, files_to_write):
        # Write files left to new Zipfile
        with zipfile.ZipFile(new_package_file_path, "a") as new_zip_file:
            for file_to_write in files_to_write:
                zip_info = self._package_file.getinfo(file_to_write)
                LOGGER.debug('Writing file "%s"', file_to_write)
                new_zip_file.writestr(
                    file_to_write,
                    self._package_file.read(file_to_write),
                    zip_info.compress_type,
                )

    def _get_optimise_web_xml(self, xml_filename, xml_related_files):
        image_filenames = [
            filename
            for filename in xml_related_files
            if not os.path.splitext(filename)[-1] == ".pdf"
        ]
        return XMLWebOptimiser(
            xml_filename,
            image_filenames,
            self._read_file,
            self._extracted_package,
            self._stop_if_error,
        )

    def _read_file(self, image_to_optimise):
        try:
            image_bytes = self._package_file.read(image_to_optimise)
        except KeyError as exc:
            raise exceptions.SPPackageError(
                "No file named {} in package".format(image_to_optimise)
            )
        else:
            return image_bytes

    def optimise(self, new_package_file_path=None, preserve_files=True):
        """Optimise SciELO Publishing Package to have WEB images alternatives.

        For each XML file in package, optimise XML with WEB images alternatives.
        In the end, creates a new SciELO Publishing Package, compressed in a ZIP file,
        with previous content and updates all optimised XMLs and the web images
        versions.
        
        :param new_package_file_path (default=None): Path to optimised SciELO Publishing
            Package file. If not given, it will be the same path and file name of the
            original package file ended with ``_optimised.zip``.
        :param preserve_files (default=True): preserve extracted and optimised files in
            aux directory. If False, it will delete files after written in new Package.
        """
        if new_package_file_path is None:
            new_package_file_path = self._extracted_package + "_optimised.zip"
        LOGGER.info(
            "Generating new SciELO Publishing Package %s", new_package_file_path
        )
        zipped_filenames = self._package_file.namelist()
        xmls_filenames = [
            xml_filename
            for xml_filename in zipped_filenames
            if os.path.splitext(xml_filename)[-1] == ".xml"
        ]
        optimised_filenames = []
        for i, xml_filename in enumerate(xmls_filenames):
            LOGGER.info(
                "Optimizing XML file %s [%s/%s]", xml_filename, i, len(xmls_filenames)
            )
            filename_root, __ = os.path.splitext(xml_filename)
            optimised_filenames += self._optimise_to_zipfile(
                new_package_file_path, xml_filename, zipped_filenames
            )

        LOGGER.info(
            "Writing remained files from package in new SciELO Publishing Package"
        )
        self._write_files_left(
            new_package_file_path, set(zipped_filenames) - set(optimised_filenames)
        )

        if preserve_files:
            with zipfile.ZipFile(new_package_file_path) as new_zip_file:
                new_zip_file.extractall(self._extracted_package)



# coding: utf-8
from flask import current_app
from flask_wtf import FlaskForm
from flask_babelex import gettext as _
from wtforms import BooleanField, FileField, StringField, validators


class XMLUploadForm(FlaskForm):
    add_scielo_br_rules = BooleanField(default=True)
    url_static_file = StringField(label=_("URL to statics files"))
    file = FileField(label=_("File"), validators=[validators.input_required()])

    def validate_file(form, field):
        _file = field.data
        if _file:
            if _file.content_type not in ["text/xml", "application/xml"]:
                raise validators.ValidationError(
                    _(u"This type of file is not allowed! Please select another file.")
                )

            # _data = _file.read()
            # if len(_data) > current_app.config.get('SETTINGS_MAX_UPLOAD_SIZE', 0):
            #     raise validators.ValidationError(_(u"The file's size is too large! Please select a smaller file."))



# coding:utf-8
"""Adapters for XML errors.

An adapter is an object that provides domain-specific apis for another object,
called adaptee.
"""
from __future__ import unicode_literals
import io
import re
import logging

from lxml import etree


LOGGER = logging.getLogger(__name__)


EXPOSE_ELEMENTNAME_PATTERN = re.compile(r"(?<=Element )'.*?'")


def search_element_name(message):
    """Try to locate in `message` the element name pointed as error.

    :param message: is a lxml error log message.
    """
    match = EXPOSE_ELEMENTNAME_PATTERN.search(message)
    if match is None:
        LOGGER.info('cannot find the element name in message')
        LOGGER.debug('failed regexp was %s on message "%s"', 
                EXPOSE_ELEMENTNAME_PATTERN.pattern, message)
        raise ValueError('cannot find the element name in message')

    else:
        element_name = match.group(0).strip("'")
        LOGGER.info('found element name "%s" in message', element_name)

        return element_name


def search_element(doc, xpath, line=None):
    """Try to locate in `doc` the element expressed as `xpath`.
    """
    for elem in doc.xpath(xpath):
        if line is None:
            return elem

        elif elem.sourceline == line:
            return elem

        else:
            continue

    # raise ValueError if the element could not be located.
    LOGGER.info('could not find element "%s"', xpath)
    raise ValueError('could not find element "%s"' % xpath)


#--------------------------------
# adapters for XML style errors
#--------------------------------
class StyleErrorBase(object):
    """Acts like an interface for SPS style errors.

    A basic implementation of `get_apparent_element` is provided.
    """
    line = message = level = None
    label = u''

    def get_apparent_element(self, doc):
        """The apparent element presenting the error at doc.

        This base implementation tries to discover the element name by
        searching the string pattern `Element 'element name'` on message.
        """
        return NotImplemented


class StyleError(StyleErrorBase):
    """ SciELO-style errors raised by the validation pipeline.
    """
    level = u'Style Error'

    def get_apparent_element(self, doc):
        """The apparent element presenting the error at doc.

        This base implementation tries to discover the element name by
        searching the string pattern `Element 'element name'` on message.
        """
        tagname = search_element_name(self.message)
        return search_element(doc, '//' + tagname, line=self.line)


class SchemaStyleError(StyleErrorBase):
    """ DTD errors.
    """
    level = u'DTD Error'

    def __init__(self, err_object, label=u''):
        self._err = err_object
        self.message = self._err.message
        self.line = self._err.line
        self.label = label

    def get_apparent_element(self, doc):
        for elem in doc.iter():
            if elem.sourceline == self.line:
                return elem

        LOGGER.info("cannot find element at the line %s", self.line)
        raise ValueError("cannot find element at the line %s" % self.line)


class SchematronStyleError(StyleErrorBase):
    """ SciELO-style errors raised by schematron validation.
    """
    level = u'Style Error'

    def __init__(self, err_object, label=u''):
        self._err = err_object
        self.label = label

        byte_string = io.BytesIO(self._err.message.encode('utf-8'))
        self._parsed_message = etree.parse(byte_string)

    @property
    def message(self):
        search_res = re.search(r"<svrl:text>(.*)</svrl:text>",
                self._err.message, re.DOTALL)
        try:
            text = search_res.group(1)
        except AttributeError:
            raise ValueError('cannot get message')

        return text.strip()

    def get_apparent_element(self, doc):
        query_res = self._parsed_message.xpath('@location')
        try:
            tagname = query_res[0]
        except IndexError:
            raise ValueError('cannot get context info')

        return search_element(doc, tagname)




# coding: utf-8
from __future__ import print_function, unicode_literals
import os
import argparse
import sys
import pkg_resources
import json
import logging
import pathlib

from lxml import etree

import packtools
from packtools import exceptions
from packtools import catalogs

__all__ = ['summarize', 'annotate']


LOGGER = logging.getLogger(__name__)


LOGGER_FMT = '%(asctime)s [%(levelname)s] %(name)s: %(message)s'


EPILOG = """\
Copyright 2013 SciELO <scielo-dev@googlegroups.com>.
Licensed under the terms of the BSD license. Please see LICENSE in the source
code for more information.
"""


ERR_MESSAGE = "Something went wrong while working on {filename}: {details}."


AVAILABLE_SCHEMAS = ', '.join(sorted(['@'+key 
    for key in catalogs.SCH_SCHEMAS.keys()]))


def get_xmlvalidator(xmlpath, no_network, extra_sch):
    """ Get an instance of ``packtools.XMLValidator``.

    :param xmlpath: filesystem or URL to an XML file.
    :param no_network: if the parser might retrieve the DTD from the internet.
    :param extra_sch: list of paths to schematron schemas.
    """ 
    parsed_xml = packtools.XML(xmlpath, no_network=no_network)
    _extra_sch = list(extra_sch)
    if _extra_sch:
        paths = [packtools.utils.resolve_schematron_filepath(path_or_ref)
                 for path_or_ref in _extra_sch]
        schemas = [packtools.utils.get_schematron_from_filepath(path)
                   for path in paths]
        labeled_schemas = zip(schemas, _extra_sch)
    else:
        labeled_schemas = None

    return packtools.XMLValidator.parse(parsed_xml, 
            extra_sch_schemas=labeled_schemas)


def annotate(validator, buff, encoding=None):
    _encoding = encoding or validator.encoding
    err_xml = validator.annotate_errors()

    buff.write(etree.tostring(err_xml, pretty_print=True,
        encoding=_encoding, xml_declaration=True))


def summarize(validator, assets_basedir=None):
    """Produce a summarized result of the validation.
    """
    def _make_err_message(err):
        """ An error message is comprised of the message itself and the
        element sourceline.
        """
        err_msg = {'message': err.message}

        try:
            err_element = err.get_apparent_element(validator.lxml)
        except ValueError:
            LOGGER.info('could not find the element name in message')
            err_element = None

        if err_element is not None:
            err_msg['apparent_line'] = err_element.sourceline
        else:
            err_msg['apparent_line'] = None

        return err_msg

    dtd_is_valid, dtd_errors = validator.validate()
    sps_is_valid, sps_errors = validator.validate_style()

    summary = {
        'dtd_errors': [_make_err_message(err) for err in dtd_errors],
        'style_errors': {},
        'is_valid': bool(dtd_is_valid and sps_is_valid),
    }

    for sps_error in sps_errors:
        err_group = summary['style_errors'].setdefault(sps_error.label, [])
        err_group.append(_make_err_message(sps_error))

    if assets_basedir:
        LOGGER.info('starting to look for assets')
        summary['assets'] = validator.lookup_assets(assets_basedir)
        LOGGER.info('total assets referenced: %s', len(summary['assets']))

    return summary


def _make_relative_to_base(base, paths):
    for path in paths:
        # pure paths don't access the filesystem
        posix_path = pathlib.PurePosixPath(path)
        try:
            relative_path = posix_path.relative_to(base)
        except ValueError:
            continue

        yield str(relative_path)


def validate_zip_package(filepath):
    """Validates all documents in a zip package.

    Returns a generator object that produces validation reports for each
    XML document. Validation reports are represented as 5-tuples in the form:
    (<filename>, <is_valid>, <validation_errors>, <exc_type>, <exc_value>)
    """
    with packtools.utils.Xray.fromfile(filepath) as xpack:
        xmls = xpack.show_sorted_members().get('xml', [])

        for xml in xmls:
            # useful for looking-up files relative to the xml file
            xml_dirname = os.path.dirname(xml)

            with xpack.get_file(xml) as file:
                try:
                    validator = packtools.XMLValidator.parse(file)

                except exceptions.PacktoolsError as exc:
                    exc_type = type(exc).__name__
                    exc_value = str(exc)
                    summary = None

                else:
                    exc_type = None
                    exc_value = None

                    paths = _make_relative_to_base(xml_dirname,
                            xpack.show_members())
                    summary = summarize(validator, paths)

            yield (xml, summary, exc_type, exc_value)


@packtools.utils.config_xml_catalog
def _main():
    exit_status = 0

    packtools_version = pkg_resources.get_distribution('packtools').version

    parser = argparse.ArgumentParser(
            description='SciELO PS stylechecker command line utility.',
            epilog=EPILOG)

    mutex_group = parser.add_mutually_exclusive_group()
    mutex_group.add_argument('--annotated', action='store_true',
                             help='reproduces the XML with notes at elements that have errors')
    mutex_group.add_argument('--raw', action='store_true',
                             help='each result is encoded as json, without any formatting, and written to stdout in a single line.')

    parser.add_argument('--nonetwork', action='store_true',
                        help='prevents the retrieval of the DTD through the network')
    parser.add_argument('--assetsdir', default=None,
                        help='lookup, at the given directory, for each asset referenced by the XML. current working directory will be used by default.')
    parser.add_argument('--version', action='version', version=packtools_version)
    parser.add_argument('--loglevel', default='')  # disabled by default
    parser.add_argument('--nocolors', action='store_false',
                        help='prevents the output from being colorized by ANSI escape sequences')
    parser.add_argument('--extrasch', action='append', default=[],
                        help='runs an extra validation using an external schematron schema. built-in schemas are available through the prefix `@`: %s.' % AVAILABLE_SCHEMAS)
    parser.add_argument('--sysinfo', action='store_true',
                        help='show program\'s installation info and exit.')
    parser.add_argument('file', nargs='*',
                        help='filesystem path or URL to the XML')
    args = parser.parse_args()

    # All log messages will be omited if level > 50
    logging.basicConfig(level=getattr(logging, args.loglevel.upper(), 999),
            format=LOGGER_FMT)

    if args.sysinfo:
        print(packtools.utils.prettify(packtools.get_debug_info(), colorize=args.nocolors))
        sys.exit(0)

    print('Please wait, this may take a while...', file=sys.stderr)

    input_args = args.file or sys.stdin
    summary_list = []

    LOGGER.info('running with catalog: %s', catalogs.NAME)

    for xml in packtools.utils.flatten(input_args):
        LOGGER.info('starting validation of "%s"', xml)

        try:
            validator = get_xmlvalidator(xml, args.nonetwork, args.extrasch)

        except (etree.XMLSyntaxError, exceptions.XMLDoctypeError,
                exceptions.XMLSPSVersionError) as exc:
            LOGGER.exception(exc)
            print(ERR_MESSAGE.format(filename=xml, details=exc),
                    file=sys.stderr)
            exit_status = 1
            continue

        if args.annotated:

            fname, fext = xml.rsplit('.', 1)
            out_fname = '.'.join([fname, 'annotated', fext])

            with open(out_fname, 'wb') as fp:
                annotate(validator, fp)

            is_valid, _ = validator.validate_all()
            if is_valid is False:
                exit_status = 1

            print('Annotated XML file: "%s"' % out_fname)

        else:
            # remote XML will not lookup for assets
            if xml.startswith(('http:', 'https:')):
                LOGGER.info('disabling assets lookup since "%s" is a '
                            'remote file', xml)
                assetsdir = ''
                assetsdir_files = []
            else:
                assetsdir = args.assetsdir or os.path.dirname(xml)
                assetsdir_files = os.listdir(assetsdir)  # list of files in dir

            try:
                summary = summarize(validator, assets_basedir=assetsdir_files)
            except TypeError as exc:
                LOGGER.exception(exc)
                LOGGER.info(
                        'error validating "%s". Skipping. '
                        'run with option `--loglevel INFO` for more info',
                        xml)
                continue

            summary['_xml'] = xml

            if args.raw:
                print(json.dumps(summary, sort_keys=True))
            else:
                summary_list.append(summary)

            # set the exit status to 1 if the xml is not valid
            if summary['is_valid'] is False:
                exit_status = 1

        LOGGER.info('finished validating "%s"', xml)

    if summary_list:
        print(packtools.utils.prettify(summary_list, colorize=args.nocolors))

    return exit_status


def main():
    try:
        sys.exit(_main())
    except KeyboardInterrupt:
        LOGGER.info('terminating the program')
        sys.exit(1)
    except Exception as exc:
        LOGGER.exception(exc)
        sys.exit('An unexpected error has occurred: %s' % exc)


if __name__ == '__main__':
    main()




#!/usr/bin/env python
#coding:utf-8
from __future__ import unicode_literals
from setuptools import setup
import setuptools
import codecs
import sys


if sys.version_info[0:2] < (2, 7):
    raise RuntimeError('Requires Python 2.7 or newer')


# adds version to the local namespace
VERSION = {}
with open('packtools/version.py') as fp:
    exec(fp.read(), VERSION)


INSTALL_REQUIRES = [
    'lxml>=4.2.0',
    'picles.plumber>=0.11',
    'Pillow~=6.2',
]


EXTRAS_REQUIRE = {
    'webapp':[
        'Flask',
        'Flask-BabelEx',
        'Flask-WTF',
        'Werkzeug==0.16.1',
    ]
}


TESTS_REQUIRE = [
    'Flask-Testing>=0.6.2',
    'Flask-BabelEx',
    'Flask-WTF',
]


# from https://hynek.me/articles/conditional-python-dependencies/
# basically, binary wheels built using setuptools version < 18 do not support
# the syntax for conditional dependencies used below.
if int(setuptools.__version__.split('.', 1)[0]) < 18:
    assert "bdist_wheel" not in sys.argv, "setuptools 18 required for wheels."
    if sys.version_info[0:2] < (3, 4):
        INSTALL_REQUIRES.append('pathlib>=1.0.1')
else:
    EXTRAS_REQUIRE[':python_version<"3.4"'] = ['pathlib>=1.0.1']

if sys.version_info[0:2] == (2, 7):
    TESTS_REQUIRE.append('mock==3.0.5')

setup(
    name="packtools",
    version=VERSION['__version__'],
    description="Handle SPS packages like a breeze.",
    long_description=codecs.open('README.md', mode='r', encoding='utf-8').read() + '\n\n' +
                     codecs.open('HISTORY.md', mode='r', encoding='utf-8').read(),
    long_description_content_type="text/markdown",
    author="SciELO",
    author_email="scielo-dev@googlegroups.com",
    maintainer="Gustavo Fonseca",
    maintainer_email="gustavo.fonseca@scielo.org",
    license="BSD License",
    url="http://docs.scielo.org",
    packages=setuptools.find_packages(
        exclude=["*.tests", "*.tests.*", "tests.*", "tests", "docs"]
    ),
    include_package_data=True,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    tests_require=TESTS_REQUIRE,
    test_suite='tests',
    install_requires=INSTALL_REQUIRES,
    extras_require=EXTRAS_REQUIRE,
    entry_points={
        "console_scripts":[
            "stylechecker=packtools.stylechecker:main",
            "htmlgenerator=packtools.htmlgenerator:main",
            "package_optimiser=packtools.package_optimiser:main",
        ]
    }
)
